/* Test client for SSL connection to FreeVSD.
   Written by Nick Burrett <nick@dsvr.net>  */

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

#define NO_IDEA
#include <openssl/crypto.h>
#include <openssl/x509.h>
#include <openssl/pem.h>
#include <openssl/ssl.h>
#include <openssl/err.h>

int main (int argc, char *argv[])
{
  SSL_CTX *ctx;
  SSL *ssl;
  X509 *server_cert;
  char buf[4096];
  int err, sd;
  struct sockaddr_in sa;

  if (argc >= 3)
    {
      fprintf (stderr, "syntax: client [<protocol file>]\n");
      fprintf (stderr, "usage: without an argument client simply tests the SSL connection\n");
      fprintf (stderr, "       with an argument, protocol commands are taken from <file>\n");
      return 1;
    }

  SSLeay_add_ssl_algorithms ();
  SSL_load_error_strings ();
  ctx = SSL_CTX_new (SSLv23_method ());
  if (ctx == NULL)
    {
      fprintf (stderr, "error: %s\n",
	       ERR_error_string (ERR_get_error (), buf));
      return 1;
    }

  if (SSL_CTX_use_PrivateKey_file (ctx,
				   SSL_BASE "/client/test1.key",
					 SSL_FILETYPE_PEM) <= 0
      || SSL_CTX_use_certificate_file (ctx,
				       SSL_BASE "/client/test1.crt",
				       SSL_FILETYPE_PEM) <= 0)
    {
      fprintf (stderr, "error: %s\n",
	       ERR_error_string (ERR_get_error (), buf));
      return 1;
    }

  /* Check the private key is valid.  */
  if (! SSL_CTX_check_private_key (ctx))
    {
      fprintf (stderr,
	       "error: Private key does not match the certificate public key\n");
      return 1;
    }

  sd = socket (AF_INET, SOCK_STREAM, 0);
  memset (&sa, 0, sizeof (sa));
  sa.sin_family = AF_INET;
  sa.sin_addr.s_addr = inet_addr ("127.0.0.1");
  sa.sin_port = htons (1725);
  connect (sd, (struct sockaddr *) &sa, sizeof (sa));

  /* SSL_CTX_set_verify (ctx, SSL_VERIFY_PEER, NULL); */

  ssl = SSL_new (ctx);
  SSL_set_fd (ssl, sd);
  SSL_connect (ssl);

  printf ("SSL connection using %s\n", SSL_get_cipher (ssl));
  server_cert = SSL_get_peer_certificate (ssl);
  if (server_cert != NULL)
    {
      char *s, buf[256];

      printf ("server certificate:\n");
      s = X509_NAME_oneline (X509_get_subject_name (server_cert),
			     buf, sizeof (buf));
      if (s)
        printf ("  subject: %s\n", s);
      s = X509_NAME_oneline (X509_get_issuer_name (server_cert),
			     buf, sizeof (buf));
      if (s)
        printf ("  issuer: %s\n", s);

      X509_free (server_cert);
    }
  else
    printf ("server does not have a certificate\n");

  err = SSL_read (ssl, buf, sizeof (buf) - 1);
  buf[err] = '\0';
  printf ("Returned: %s\n", buf);
  if (argc == 1)
    err = SSL_write (ssl, "EOF\n", strlen ("EOF\n"));
  else
    {
      FILE *handle = fopen (argv[1], "r");
      if (handle != NULL)
	{
	  char line[256];

	  while (fgets (line, sizeof (line) - 1, handle))
	    err = SSL_write (ssl, line, strlen (line));

	  fclose (handle);
	}
      else
	{
	  err = SSL_write (ssl, "EOF\n", strlen ("EOF\n"));
	}
    }
  while ((err = SSL_read (ssl, buf, sizeof (buf) - 1)))
    {
      buf[err] = '\0';
      printf ("Returned: %s\n", buf);
    }
  SSL_shutdown (ssl);

  close (sd);
  SSL_free (ssl);
  SSL_CTX_free (ctx);

  return 0;
}
