/* Module command interface.
   Copyright (c) 1999 Idaya Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "vsd.h"
#include "module.h"
#include "apache.h"
#include "mod_apache.h"

struct command_tag mod_apache_command_table[] =
{
  {
    "APACHE_VHENUM", CMD_APACHE_VHENUM, ADMIN_VS,
    "List all configured VirtualHost declarations", 0, "",
    NULL, 0, NULL
  },
  {
    "APACHE_VHADD", CMD_APACHE_VHADD, ADMIN_VS,
    "Add configuration for a virtual host", 0, "",
    NULL, 0, NULL
  },
  {
    "APACHE_VHDELETE", CMD_APACHE_VHDELETE, ADMIN_VS,
    "Delete a VirtualHost declaration", 2, "<virtual-host> <server-name>",
    NULL, 0, NULL
  },
  {
    "APACHE_VHMODIFY", CMD_APACHE_VHMODIFY, ADMIN_VS,
    "Modify a VirtualHost declaration", 2, "<virtual-host> <server-name>",
    NULL, 0, NULL
  },
  {
    "APACHE_VHGET", CMD_APACHE_VHGET, ADMIN_VS,
    "Return the contents of a VirtualHost declaration",
    2, "<virtual-host> <server-name>",
    NULL, 0, NULL
  },
  {
    NULL, 0, 0,
    NULL, 0, NULL,
    NULL, 0, NULL
  } /* Terminator */
};

/* Function prototypes */

const char *mod_apache_check_syntax (int level, const char *command,
				     int argc, char *argv[])
{
  struct command_tag *list;

  list = module_lookup_command (mod_apache_command_table, level, command);

  /* Command doesn't exist.  */
  if (! list)
    return "notexist";

  if (argc != list->num_args)
    return list->syntax;

  return NULL;
}

/* Return 0 if command processed, 1 if not recognised.  */

int mod_apache_transaction (struct connection *vc,
			    int level, const char *command,
			    int argc, char *argv[], const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (mod_apache_command_table, level, command);

  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_APACHE_VHENUM:
      httpd_enumerate_virtual_hosts (vc, argc, argv);
      break;
    case CMD_APACHE_VHADD:
      httpd_add_virtual_host (vc, argc, argv, data);
      break;
    case CMD_APACHE_VHDELETE:
      httpd_delete_virtual_host (vc, argc, argv);
      break;
    case CMD_APACHE_VHMODIFY:
      httpd_modify_virtual_host (vc, argc, argv, data);
      break;
    case CMD_APACHE_VHGET:
      httpd_get_virtual_host (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}

int mod_apache_help (int level, const char *command)
{
  return 0;
}

module apache_module =
{
  mod_apache_check_syntax,
  mod_apache_help,
  mod_apache_transaction
};
