/* Module command interface.
   Copyright (c) 1999 Idaya Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "vsd.h"
#include "module.h"
#include "priv.h"
#include "mod_priv.h"

struct command_tag mod_priv_command_table[] =
{
  {
    "PRIV_MODIFY", CMD_PRIV_MODIFY, ADMIN_VS,
    "Set privileges for a user on a virtual server",
    2, "<user> <+|-><privilege>",
    NULL, 0, NULL
  },
  {
    "PRIV_LIST", CMD_PRIV_LIST, ADMIN_VS,
    "List privileges that are assigned to a user", 0, "[<user>]",
    NULL, 0, NULL
  },
  {
    "PRIV_AVAIL", CMD_PRIV_AVAIL, ADMIN_VS,
    "List all available privileges on a virtual server", 0, "",
    NULL, 0, NULL
  },
  {
    NULL, 0, 0,
    NULL, 0, NULL,
    NULL, 0, NULL
  } /* Terminator */
};

/* Function prototypes */

const char *mod_priv_check_syntax (int level, const char *command,
				   int argc, char *argv[])
{
  struct command_tag *list;

  list = module_lookup_command (mod_priv_command_table, level, command);

  /* Command doesn't exist.  */
  if (! list)
    return "notexist";

  if (argc < list->num_args)
    return list->syntax;

  return NULL;
}

/* Return 0 if command processed, 1 if not recognised.  */
int mod_priv_transaction (struct connection *vc,
			  int level, const char *command,
			  int argc, char *argv[], const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (mod_priv_command_table, level, command);

  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_PRIV_MODIFY:
      priv_modify (vc, argc, argv);
      break;
    case CMD_PRIV_LIST:
      priv_list (vc, argc, argv);
      break;
    case CMD_PRIV_AVAIL:
      priv_avail (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}

int mod_priv_help (int level, const char *command)
{
  return 0;
}

module priv_module =
{
  mod_priv_check_syntax,
  mod_priv_help,
  mod_priv_transaction
};
