/* Module command interface.
   Copyright (c) 1999, 2000 Idaya Ltd.
   Written by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD).

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "module.h"
#include "quota.h"
#include "mod_quota.h"

struct command_tag mod_quota_command_table[] =
{
  {
    "QUOTA_VSSET", CMD_QUOTA_VSSET, ADMIN_VS,
    "Set disk quota limits for a virtual server", 1, "<quota>{K|M}",
    NULL, 0, NULL
  },
  {
    "QUOTA_VSGET", CMD_QUOTA_VSGET, ADMIN_VS,
    "Get disk quota limits for a virtual server", 0, "",
    NULL, 0, NULL
  },
  {
    "QUOTA_USERGET", CMD_QUOTA_USERGET, ADMIN_VS,
    "Get quota limits for a user", 1, "<user>",
    NULL, 0, NULL
  },
  {
    "QUOTA_USERSET", CMD_QUOTA_USERSET, ADMIN_VS,
    "Set quota limits for a user", 2, "<user> [+|-]size{K|M}",
    NULL, 0, NULL
  },
  {
    "QUOTA_STATS", CMD_QUOTA_STATS, ADMIN_VS,
    "Get quota details for all users on a server", 0, "",
    NULL, 0, NULL
  },
  {
    NULL, 0, 0,
    NULL, 0, NULL,
    NULL, 0, NULL
  } /* Terminator */
};

/* Function prototypes */

const char *mod_quota_check_syntax (int level, const char *command,
				    int argc, char *argv[])
{
  struct command_tag *list;

  list = module_lookup_command (mod_quota_command_table, level, command);

  /* Command doesn't exist.  */
  if (! list)
    return "notexist";

  if (argc != list->num_args)
    return list->syntax;

  return NULL;
}

/* Return 0 if command processed, 1 if not recognised.  */

int mod_quota_transaction (struct connection *vc, int level,
			   const char *command, int argc, char *argv[],
			   const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (mod_quota_command_table, level, command);

  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_QUOTA_VSSET:
      quota_vsset (vc, argc, argv);
      break;
    case CMD_QUOTA_VSGET:
      quota_vsget (vc, argc, argv);
      break;
    case CMD_QUOTA_USERGET:
      quota_userget (vc, argc, argv);
      break;
    case CMD_QUOTA_USERSET:
      quota_userset (vc, argc, argv);
      break;
    case CMD_QUOTA_STATS:
      quota_stats (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}

int mod_quota_help (int level, const char *command)
{
  return 0;
}

module quota_module =
{
  mod_quota_check_syntax,
  mod_quota_help,
  mod_quota_transaction
};
