/* Module command interface.
   Copyright (c) 1999 Idaya Ltd.
   Written by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "module.h"
#include "sendmail.h"
#include "mod_sendmail.h"

struct command_tag mod_sendmail_command_table[] =
{
  {
    "SENDMAIL_VLIST", CMD_SENDMAIL_VLIST, ADMIN_VS,
    "List e-mail aliases for a domain", 1, "<domain-name>",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_VMODIFY", CMD_SENDMAIL_VMODIFY, ADMIN_VS,
    "Modify e-mail aliases", 0, "",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_CWLIST", CMD_SENDMAIL_CWLIST, ADMIN_VS,
    "List domains configured in sendmail.cw", 0, "",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_CWADD", CMD_SENDMAIL_CWADD, ADMIN_VS,
    "Add a domain to sendmail.cw", 1, "<domain-name>",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_CWDELETE", CMD_SENDMAIL_CWDELETE, ADMIN_VS,
    "Delete a domain from sendmail.cw", 1, "<domain-name>",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_ALISTLHS", CMD_SENDMAIL_ALISTLHS, ADMIN_VS,
    "List LHS entries in /etc/aliases", 0, "",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_AGETRHS", CMD_SENDMAIL_AGETRHS, ADMIN_VS,
    "Get RHS entries in /etc/aliases", 1, "<lhs>",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_AADD", CMD_SENDMAIL_AADD, ADMIN_VS,
    "Add an entry to /etc/aliases", 2, "<lhs> <rhs>",
    NULL, 0, NULL
  },
  {
    "SENDMAIL_ADELETE", CMD_SENDMAIL_ADELETE, ADMIN_VS,
    "Delete an entry from /etc/aliases", 1, "<domain-name>",
    NULL, 0, NULL
  },
  {
    NULL, 0, 0,
    NULL, 0, NULL,
    NULL, 0, NULL
  } /* Terminator */
};

/* Function prototypes */

const char *mod_sendmail_check_syntax (int level, const char *command,
				       int argc, char *argv[])
{
  struct command_tag *list;

  list = module_lookup_command (mod_sendmail_command_table, level, command);

  /* Command doesn't exist.  */
  if (! list)
    return "notexist";

  if (argc != list->num_args)
    return list->syntax;

  return NULL;
}

/* Return 0 if command processed, 1 if not recognised.  */

int mod_sendmail_transaction (struct connection *vc,
			      int level, const char *command,
			      int argc, char *argv[], const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (mod_sendmail_command_table, level, command);

  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_SENDMAIL_VLIST:
      sendmail_virt_list_domain (vc, argc, argv);
      break;
    case CMD_SENDMAIL_VMODIFY:
      sendmail_virt_modify_table (vc, argc, argv, data);
      break;
    case CMD_SENDMAIL_CWLIST:
      sendmail_cw_list_domain (vc, argc, argv);
      break;
    case CMD_SENDMAIL_CWADD:
      sendmail_cw_add_domain (vc, argc, argv);
      break;
    case CMD_SENDMAIL_CWDELETE:
      sendmail_cw_delete_domain (vc, argc, argv);
      break;
    case CMD_SENDMAIL_ALISTLHS:
      sendmail_alistlhs (vc, argc, argv);
      break;
    case CMD_SENDMAIL_AGETRHS:
      sendmail_agetrhs (vc, argc, argv);
      break;
    case CMD_SENDMAIL_AADD:
      sendmail_aadd (vc, argc, argv);
      break;
    case CMD_SENDMAIL_ADELETE:
      sendmail_adelete (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}

int mod_sendmail_help (int level, const char *command)
{
  return 0;
}

module sendmail_module =
{
  mod_sendmail_check_syntax,
  mod_sendmail_help,
  mod_sendmail_transaction
};
