/* Module command interface.
   Copyright (c) 1999, 2000 Idaya Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "vsd.h"
#include "module.h"
#include "user.h"
#include "mod_user.h"

struct command_tag mod_user_command_table[] =
{
  {
    "USER_ADD", CMD_USER_ADD, ADMIN_VS,
    "Create a new user", 1, "<user name>",
    NULL, 0, NULL
  },
  {
    "USER_DEL", CMD_USER_DEL, ADMIN_VS,
    "Delete a user", 1, "<user name>",
    NULL, 0, NULL
  },
  {
    "USER_MOD", CMD_USER_MODIFY, ADMIN_VS,
    "Modify a user account", 1, "<user name>",
    NULL, 0, NULL
  },
  {
    "USER_LIST", CMD_USER_LIST, ADMIN_VS,
    "List all users on a virtual server", 0, "",
    NULL, 0, NULL
  },
  {
    "USER_CHPASS", CMD_USER_CHPASS, ADMIN_VS,
    "Change the password for a user", 3, "<user> <old pass> <new pass>",
    NULL, 0, NULL
  },
  {
    "USER_GRADD", CMD_USER_GRADD, ADMIN_VS,
    "Create a new group", 1, "<group>",
    NULL, 0, NULL
  },
  {
    "USER_GRDEL", CMD_USER_GRDEL, ADMIN_VS,
    "Delete a group", 1, "<group>",
    NULL, 0, NULL
  },
  {
    "USER_GRMOD", CMD_USER_GRMOD, ADMIN_VS,
    "Add/remove a user from a group", 2, "<group> <+|-><user>",
    NULL, 0, NULL
  },
  {
    "USER_GRLIST", CMD_USER_GRLIST, ADMIN_VS,
    "List all groups on a virtual server", 0, "",
    NULL, 0, NULL
  },
  {
    NULL, 0, 0,
    NULL, 0, NULL,
    NULL, 0, NULL
  } /* Terminator */
};

/* Function prototypes */
const char *mod_user_check_syntax (int level, const char *command,
				   int argc, char *argv[])
{
  struct command_tag *list;

  list = module_lookup_command (mod_user_command_table, level, command);

  /* Command doesn't exist.  */
  if (! list)
    return "notexist";

  if (argc < list->num_args)
    return list->syntax;

  return NULL;
}

/* Return 0 if command processed, 1 if not recognised.  */
int mod_user_transaction (struct connection *vc,
			  int level, const char *command,
			  int argc, char *argv[], const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (mod_user_command_table, level, command);

  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_USER_ADD:
      user_add (vc, argc, argv);
      break;
    case CMD_USER_DEL:
      user_del (vc, argc, argv);
      break;
    case CMD_USER_MODIFY:
      user_mod (vc, argc, argv);
      break;
    case CMD_USER_LIST:
      user_list (vc, argc, argv);
      break;
    case CMD_USER_CHPASS:
      user_chpass (vc, argc, argv);
      break;
    case CMD_USER_GRADD:
      user_group_add (vc, argc, argv);
      break;
    case CMD_USER_GRDEL:
      user_group_del (vc, argc, argv);
      break;
    case CMD_USER_GRMOD:
      user_group_mod (vc, argc, argv);
      break;
    case CMD_USER_GRLIST:
      user_group_list (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}

int mod_user_help (int level, const char *command)
{
  return 0;
}

module user_module =
{
  mod_user_check_syntax,
  mod_user_help,
  mod_user_transaction
};
