/* Module command interface.
   Copyright (c) 1999 Idaya Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "module.h"
#include "mod_vs.h"
#include "vs.h"

struct command_tag mod_vs_command_table[] =
{
  {
    "VS_CREATE", CMD_VS_CREATE, ADMIN_HOST,
    "Create a new virtual server account",
    5, "<virtual-server> <ip-addr> <fqdn> <numuid> <quota> [<ns1>] [<ns2>]",
    NULL, 0, NULL
  },
  {
    "VS_DELETE", CMD_VS_DELETE, ADMIN_HOST,
    "Delete a virtual server account", 1, "<virtual-server>",
    NULL, 0, NULL
  },
  {
    "VS_ENABLE", CMD_VS_ENABLE, ADMIN_HOST,
    "Enable a virtual server account", 1, "<virtual-server>",
    NULL, 0, NULL
  },
  {
    "VS_DISABLE", CMD_VS_DISABLE, ADMIN_HOST,
    "Disable a virtual server account", 1, "<virtual-server>",
    NULL, 0, NULL
  },
  {
    "VS_DETAILS", CMD_VS_DETAILS, ADMIN_VS | ADMIN_HOST,
    "Return config details for a virtual server", 0, "[<virtual-server>]",
    NULL, 0, NULL
  },
  {
    "VS_REBOOT", CMD_VS_REBOOT, ADMIN_VS,
    "Restart services on a virtual server", 0, "",
    NULL, 0, NULL
  },
  {
    "VS_SIGNAL", CMD_VS_SIGNAL, ADMIN_VS,
    "Send a signal to all processes", 1, "<signal number>",
    NULL, 0, NULL
  },
  {
    "VS_MODIFY", CMD_VS_MODIFY, ADMIN_HOST,
    "Modify characteristics of a virtual server", 6,
    "<old-vs> <new-vs> <old-fqdn> <new-fqdn> <old-ip> <new-ip>",
    NULL, 0, NULL
  },
  {
    NULL, 0, 0,
    NULL, 0, NULL,
    NULL, 0, NULL
  } /* Terminator */
};

/* Function prototypes */

const char *mod_vs_check_syntax (int level, const char *command,
				 int argc, char *argv[])
{
  struct command_tag *list;

  list = module_lookup_command (mod_vs_command_table, level, command);
  /* Command doesn't exist.  */
  if (! list)
    return "notexist";

  if (argc < list->num_args)
    return list->syntax;

  return NULL;
}

/* Return 0 if command processed, 1 if not recognised.  */

int mod_vs_transaction (struct connection *vc, int level, const char *command,
			int argc, char *argv[], const char *data)
{
  int token;
  struct command_tag *tag;

  tag = module_lookup_command (mod_vs_command_table, level, command);
  if (tag == NULL)
    return 1;

  token = tag->token;
  switch (token)
    {
    case CMD_VS_CREATE:
      vs_account_create (vc, argc, argv);
      break;
    case CMD_VS_DELETE:
      vs_account_delete (vc, argc, argv);
      break;
    case CMD_VS_ENABLE:
      vs_account_enable (vc, argc, argv);
      break;
    case CMD_VS_DISABLE:
      vs_account_disable (vc, argc, argv);
      break;
    case CMD_VS_DETAILS:
      vs_details (vc, level, argc, argv);
      break;
    case CMD_VS_REBOOT:
      vs_reboot (vc, argc, argv);
      break;
    case CMD_VS_SIGNAL:
      vs_signal (vc, argc, argv);
      break;
    case CMD_VS_MODIFY:
      vs_modify (vc, argc, argv);
      break;
    default:
      return 1;
      break;
    }

  return 0;
}

int mod_vs_help (int level, const char *command)
{
  return 0;
}

module vs_module =
{
  mod_vs_check_syntax,
  mod_vs_help,
  mod_vs_transaction
};
