/* API for interfacing with the Virtual Server administrator.
   Copyright (c) 1999, 2000 Idaya Ltd.
   Contributed by Nick Burrett <nick@dsvr.net>

   This file is part of the Virtual Server Administrator (FreeVSD)

   FreeVSD is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   FreeVSD is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with FreeVSD; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.  */

#ifndef VSD_LIB_H
#define VSD_LIB_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _vsd_server
{
  int fd; /* file descriptor.  */
  int data; /* amount of data in the buffer.  */
  char *buffer; /* data buffer */
  int buffer_size; /* size of data buffer */
  int dp; /* index into data buffer.  */
  char virtual_server[32];
  int level;
} vsd_server;


extern vsd_server *vsd_open (const char *host_server,
			     const char *virtual_server,
			     const char *username, const char *password);
extern void vsd_close (vsd_server *server);
extern int vsd_reply_success (vsd_server *server);
extern char *vsd_process_reply (vsd_server *server);
extern char *vsd_get_line (vsd_server *server);
extern int vsd_send_line (vsd_server *server, const char *fmt, ...);
extern int vsd_send_file (vsd_server *server, const char *file);
extern int vsd_send_command (vsd_server *server, const char *data,
			     const char *fmt, ...);
extern int vsd_send_command_file (vsd_server *server, const char *fname,
				  const char *fmt, ...);
extern int vsd_gethostservername (char *buffer, int size);

#ifdef __cplusplus
}
#endif

#endif
